"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    window_id: str,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the targets registered with the maintenance window.

    Args:
        window_id(str): The ID of the maintenance window whose targets should be retrieved.
        filters(List, optional): Optional filters that can be used to narrow down the scope of the returned window targets. The supported filter keys are Type, WindowTargetId, and OwnerInformation. Defaults to None.
        max_results(int, optional): The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. Defaults to None.
        next_token(str, optional): The token for the next set of items to return. (You received this token from a previous call.). Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.maintenance_window.target.describe_all(ctx, window_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.maintenance_window.target.describe_all window_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="describe_maintenance_window_targets",
        op_kwargs={
            "WindowId": window_id,
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )


async def update(
    hub,
    ctx,
    window_id: str,
    window_target_id: str,
    targets: List = None,
    owner_information: str = None,
    name: str = None,
    description: str = None,
    replace: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the target of an existing maintenance window. You can change the following:   Name   Description   Owner   IDs for an ID target   Tags for a Tag target   From any supported tag type to another. The three supported tag types are ID target, Tag target, and resource group. For more information, see Target.    If a parameter is null, then the corresponding field isn't modified.

    Args:
        window_id(str): The maintenance window ID with which to modify the target.
        window_target_id(str): The target ID to modify.
        targets(List, optional): The targets to add or replace. Defaults to None.
        owner_information(str, optional): User-provided value that will be included in any Amazon CloudWatch Events events raised while running tasks for these targets in this maintenance window. Defaults to None.
        name(str, optional): A name for the update. Defaults to None.
        description(str, optional): An optional description for the update. Defaults to None.
        replace(bool, optional): If True, then all fields that are required by the RegisterTargetWithMaintenanceWindow operation are also required for this API request. Optional fields that aren't specified are set to null. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.maintenance_window.target.update(
                ctx, window_id=value, window_target_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.maintenance_window.target.update window_id=value, window_target_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="update_maintenance_window_target",
        op_kwargs={
            "WindowId": window_id,
            "WindowTargetId": window_target_id,
            "Targets": targets,
            "OwnerInformation": owner_information,
            "Name": name,
            "Description": description,
            "Replace": replace,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def confirm(hub, ctx, confirmation_token: str) -> Dict:
    r"""
    **Autogenerated function**

    Confirms a topic rule destination. When you create a rule requiring a destination, AWS IoT sends a confirmation message to the endpoint or base address you specify. The message includes a token which you pass back when calling ConfirmTopicRuleDestination to confirm that you own or have access to the endpoint.

    Args:
        confirmation_token(str): The token used to confirm ownership or access to the topic rule confirmation URL.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.confirm(
                ctx, confirmation_token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.confirm confirmation_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="confirm_topic_rule_destination",
        op_kwargs={"confirmationToken": confirmation_token},
    )


async def create(hub, ctx, destination_configuration: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Creates a topic rule destination. The destination must be confirmed prior to use.

    Args:
        destination_configuration(Dict): The topic rule destination configuration.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.create(
                ctx, destination_configuration=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.create destination_configuration=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="create_topic_rule_destination",
        op_kwargs={"destinationConfiguration": destination_configuration},
    )


async def delete(hub, ctx, arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a topic rule destination.

    Args:
        arn(str): The ARN of the topic rule destination to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.delete(ctx, arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.delete arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="delete_topic_rule_destination",
        op_kwargs={"arn": arn},
    )


async def get(hub, ctx, arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about a topic rule destination.

    Args:
        arn(str): The ARN of the topic rule destination.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.get(ctx, arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.get arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="get_topic_rule_destination",
        op_kwargs={"arn": arn},
    )


async def list_all(hub, ctx, max_results: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Lists all the topic rule destinations in your AWS account.

    Args:
        max_results(int, optional): The maximum number of results to return at one time. Defaults to None.
        next_token(str, optional): To retrieve the next set of results, the nextToken value from a previous response; otherwise null to receive the first set of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="list_topic_rule_destinations",
        op_kwargs={"maxResults": max_results, "nextToken": next_token},
    )


async def update(hub, ctx, arn: str, status: str) -> Dict:
    r"""
    **Autogenerated function**

    Updates a topic rule destination. You use this to change the status, endpoint URL, or confirmation URL of the destination.

    Args:
        arn(str): The ARN of the topic rule destination.
        status(str): The status of the topic rule destination. Valid values are:  IN_PROGRESS  A topic rule destination was created but has not been confirmed. You can set status to IN_PROGRESS by calling UpdateTopicRuleDestination. Calling UpdateTopicRuleDestination causes a new confirmation challenge to be sent to your confirmation endpoint.  ENABLED  Confirmation was completed, and traffic to this destination is allowed. You can set status to DISABLED by calling UpdateTopicRuleDestination.  DISABLED  Confirmation was completed, and traffic to this destination is not allowed. You can set status to ENABLED by calling UpdateTopicRuleDestination.  ERROR  Confirmation could not be completed, for example if the confirmation timed out. You can call GetTopicRuleDestination for details about the error. You can set status to IN_PROGRESS by calling UpdateTopicRuleDestination. Calling UpdateTopicRuleDestination causes a new confirmation challenge to be sent to your confirmation endpoint.  .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.topic_rule.destination.update(ctx, arn=value, status=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.topic_rule.destination.update arn=value, status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="update_topic_rule_destination",
        op_kwargs={"arn": arn, "status": status},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str,
    semantic_version: str,
    components: List,
    parent_image: str,
    client_token: str,
    description: str = None,
    block_device_mappings: List = None,
    tags: Dict = None,
    working_directory: str = None,
    additional_instance_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates a new image recipe. Image recipes define how images are configured, tested, and assessed.

    Args:
        name(str):  The name of the image recipe.
        description(str, optional):  The description of the image recipe. Defaults to None.
        semantic_version(str): The semantic version of the image recipe. This version follows the semantic version syntax.  The semantic version has four nodes: <major>.<minor>.<patch>/<build>. You can assign values for the first three, and can filter on all of them.  Assignment: For the first three nodes you can assign any positive integer value, including zero, with an upper limit of 2^30-1, or 1073741823 for each node. Image Builder automatically assigns the build number, and that is not open for updates.  Patterns: You can use any numeric pattern that adheres to the assignment requirements for the nodes that you can assign. For example, you might choose a software version pattern, such as 1.0.0, or a date, such as 2021.01.01. .
        components(List): The components of the image recipe.
        parent_image(str): The parent image of the image recipe. The value of the string can be the ARN of the parent image or an AMI ID. The format for the ARN follows this example: arn:aws:imagebuilder:us-west-2:aws:image/windows-server-2016-english-full-base-x86/x.x.x. You can provide the specific version that you want to use, or you can use a wildcard in all of the fields. If you enter an AMI ID for the string value, you must have access to the AMI, and the AMI must be in the same Region in which you are using Image Builder.
        block_device_mappings(List, optional): The block device mappings of the image recipe. Defaults to None.
        tags(Dict, optional):  The tags of the image recipe. Defaults to None.
        working_directory(str, optional): The working directory used during build and test workflows. Defaults to None.
        additional_instance_configuration(Dict, optional): Specify additional settings and launch scripts for your build instances. Defaults to None.
        client_token(str): The idempotency token used to make this request idempotent.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.imagebuilder.image.recipe.init.create(
                ctx,
                name=value,
                semantic_version=value,
                components=value,
                parent_image=value,
                client_token=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.imagebuilder.image.recipe.init.create name=value, semantic_version=value, components=value, parent_image=value, client_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="imagebuilder",
        operation="create_image_recipe",
        op_kwargs={
            "name": name,
            "description": description,
            "semanticVersion": semantic_version,
            "components": components,
            "parentImage": parent_image,
            "blockDeviceMappings": block_device_mappings,
            "tags": tags,
            "workingDirectory": working_directory,
            "additionalInstanceConfiguration": additional_instance_configuration,
            "clientToken": client_token,
        },
    )


async def delete(hub, ctx, image_recipe_arn: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes an image recipe.

    Args:
        image_recipe_arn(str): The Amazon Resource Name (ARN) of the image recipe to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.imagebuilder.image.recipe.init.delete(
                ctx, image_recipe_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.imagebuilder.image.recipe.init.delete image_recipe_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="imagebuilder",
        operation="delete_image_recipe",
        op_kwargs={"imageRecipeArn": image_recipe_arn},
    )


async def get(hub, ctx, image_recipe_arn: str) -> Dict:
    r"""
    **Autogenerated function**

     Gets an image recipe.

    Args:
        image_recipe_arn(str): The Amazon Resource Name (ARN) of the image recipe that you want to retrieve.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.imagebuilder.image.recipe.init.get(ctx, image_recipe_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.imagebuilder.image.recipe.init.get image_recipe_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="imagebuilder",
        operation="get_image_recipe",
        op_kwargs={"imageRecipeArn": image_recipe_arn},
    )


async def list_all(
    hub,
    ctx,
    owner: str = None,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of image recipes.

    Args:
        owner(str, optional): The owner defines which image recipes you want to list. By default, this request will only show image recipes owned by your account. You can use this field to specify if you want to view image recipes owned by yourself, by Amazon, or those image recipes that have been shared with you by other customers. Defaults to None.
        filters(List, optional): The filters. Defaults to None.
        max_results(int, optional): The maximum items to return in a request. Defaults to None.
        next_token(str, optional): A token to specify where to start paginating. This is the NextToken from a previously truncated response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.imagebuilder.image.recipe.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.imagebuilder.image.recipe.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="imagebuilder",
        operation="list_image_recipes",
        op_kwargs={
            "owner": owner,
            "filters": filters,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def replicate(
    hub,
    ctx,
    secret_id: str,
    add_replica_regions: List,
    force_overwrite_replica_secret: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Converts an existing secret to a multi-Region secret and begins replication the secret to a list of new regions.

    Args:
        secret_id(str): Use the Secret Id to replicate a secret to regions.
        add_replica_regions(List): Add Regions to replicate the secret.
        force_overwrite_replica_secret(bool, optional): (Optional) If set, Secrets Manager replication overwrites a secret with the same name in the destination region. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.secretsmanager.secret.to_region.replicate(
                ctx, secret_id=value, add_replica_regions=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.secretsmanager.secret.to_region.replicate secret_id=value, add_replica_regions=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="secretsmanager",
        operation="replicate_secret_to_regions",
        op_kwargs={
            "SecretId": secret_id,
            "AddReplicaRegions": add_replica_regions,
            "ForceOverwriteReplicaSecret": force_overwrite_replica_secret,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def tag(hub, ctx, resource_arn: str, tags: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Adds or edits tags for a AWS RoboMaker resource. Each tag consists of a tag key and a tag value. Tag keys and tag values are both required, but tag values can be empty strings.  For information about the rules that apply to tag keys and tag values, see User-Defined Tag Restrictions in the AWS Billing and Cost Management User Guide.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the AWS RoboMaker resource you are tagging.
        tags(Dict): A map that contains tag keys and tag values that are attached to the resource.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.resource.tag(ctx, resource_arn=value, tags=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.resource.tag resource_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="tag_resource",
        op_kwargs={"resourceArn": resource_arn, "tags": tags},
    )


async def untag(hub, ctx, resource_arn: str, tag_keys: List) -> Dict:
    r"""
    **Autogenerated function**

    Removes the specified tags from the specified AWS RoboMaker resource. To remove a tag, specify the tag key. To change the tag value of an existing tag key, use  TagResource .

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the AWS RoboMaker resource you are removing tags.
        tag_keys(List): A map that contains tag keys and tag values that will be unattached from the resource.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.resource.untag(
                ctx, resource_arn=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.resource.untag resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="untag_resource",
        op_kwargs={"resourceArn": resource_arn, "tagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def disassociate(hub, ctx, connection_id: str, lag_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Disassociates a connection from a link aggregation group (LAG). The connection is interrupted and re-established as a standalone connection (the connection is not deleted; to delete the connection, use the DeleteConnection request). If the LAG has associated virtual interfaces or hosted connections, they remain associated with the LAG. A disassociated connection owned by an Direct Connect Partner is automatically converted to an interconnect. If disassociating the connection would cause the LAG to fall below its setting for minimum number of operational connections, the request fails, except when it's the last member of the LAG. If all connections are disassociated, the LAG continues to exist as an empty LAG with no physical connections.

    Args:
        connection_id(str): The ID of the connection.
        lag_id(str): The ID of the LAG.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.directconnect.connection.from_lag.disassociate(
                ctx, connection_id=value, lag_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.directconnect.connection.from_lag.disassociate connection_id=value, lag_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="directconnect",
        operation="disassociate_connection_from_lag",
        op_kwargs={"connectionId": connection_id, "lagId": lag_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def tag(hub, ctx, resource_arn: str, tags: Dict = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates tags for a resource, given its ARN.

    Args:
        resource_arn(str): The ARN of the target resource for tagging operations.
        tags(Dict, optional): A collection of labels, in the form of key:value pairs, that apply to this resource. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.nimble.resource.tag(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.nimble.resource.tag resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="nimble",
        operation="tag_resource",
        op_kwargs={"resourceArn": resource_arn, "tags": tags},
    )


async def untag(hub, ctx, resource_arn: str, tag_keys: List) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the tags for a resource.

    Args:
        resource_arn(str): The ARN of the target resource for tagging operations.
        tag_keys(List): An array of tag keys to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.nimble.resource.untag(ctx, resource_arn=value, tag_keys=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.nimble.resource.untag resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="nimble",
        operation="untag_resource",
        op_kwargs={"resourceArn": resource_arn, "tagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str = None,
    version: str = None,
    storage_location: Dict = None,
    zip_file: bytes = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new script record for your Realtime Servers script. Realtime scripts are JavaScript that provide configuration settings and optional custom game logic for your game. The script is deployed when you create a Realtime Servers fleet to host your game sessions. Script logic is executed during an active game session.  To create a new script record, specify a script name and provide the script file(s). The script files and all dependencies must be zipped into a single file. You can pull the zip file from either of these locations:    A locally available directory. Use the ZipFile parameter for this option.   An Amazon Simple Storage Service (Amazon S3) bucket under your AWS account. Use the StorageLocation parameter for this option. You'll need to have an Identity Access Management (IAM) role that allows the Amazon GameLift service to access your S3 bucket.    If the call is successful, a new script record is created with a unique script ID. If the script file is provided as a local file, the file is uploaded to an Amazon GameLift-owned S3 bucket and the script record's storage location reflects this location. If the script file is provided as an S3 bucket, Amazon GameLift accesses the file at this storage location as needed for deployment.  Learn more   Amazon GameLift Realtime Servers   Set Up a Role for Amazon GameLift Access   Related actions   CreateScript | ListScripts | DescribeScript | UpdateScript | DeleteScript | All APIs by task

    Args:
        name(str, optional): A descriptive label that is associated with a script. Script names do not need to be unique. You can use UpdateScript to change this value later. . Defaults to None.
        version(str, optional): Version information that is associated with a build or script. Version strings do not need to be unique. You can use UpdateScript to change this value later. . Defaults to None.
        storage_location(Dict, optional): The location of the Amazon S3 bucket where a zipped file containing your Realtime scripts is stored. The storage location must specify the Amazon S3 bucket name, the zip file name (the "key"), and a role ARN that allows Amazon GameLift to access the Amazon S3 storage location. The S3 bucket must be in the same Region where you want to create a new script. By default, Amazon GameLift uploads the latest version of the zip file; if you have S3 object versioning turned on, you can use the ObjectVersion parameter to specify an earlier version. . Defaults to None.
        zip_file(bytes, optional): A data object containing your Realtime scripts and dependencies as a zip file. The zip file can have one or multiple files. Maximum size of a zip file is 5 MB. When using the AWS CLI tool to create a script, this parameter is set to the zip file name. It must be prepended with the string "fileb://" to indicate that the file data is a binary object. For example: --zip-file fileb://myRealtimeScript.zip. Defaults to None.
        tags(List, optional): A list of labels to assign to the new script resource. Tags are developer-defined key-value pairs. Tagging AWS resources are useful for resource management, access management and cost allocation. For more information, see  Tagging AWS Resources in the AWS General Reference. Once the resource is created, you can use TagResource, UntagResource, and ListTagsForResource to add, remove, and view tags. The maximum tag limit may be lower than stated. See the AWS General Reference for actual tagging limits. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.gamelift.script.create(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.gamelift.script.create
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="gamelift",
        operation="create_script",
        op_kwargs={
            "Name": name,
            "Version": version,
            "StorageLocation": storage_location,
            "ZipFile": zip_file,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, script_id: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a Realtime script. This operation permanently deletes the script record. If script files were uploaded, they are also deleted (files stored in an S3 bucket are not deleted).  To delete a script, specify the script ID. Before deleting a script, be sure to terminate all fleets that are deployed with the script being deleted. Fleet instances periodically check for script updates, and if the script record no longer exists, the instance will go into an error state and be unable to host game sessions.  Learn more   Amazon GameLift Realtime Servers   Related actions   CreateScript | ListScripts | DescribeScript | UpdateScript | DeleteScript | All APIs by task

    Args:
        script_id(str): A unique identifier for the Realtime script to delete. You can use either the script ID or ARN value.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.gamelift.script.delete(ctx, script_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.gamelift.script.delete script_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="gamelift",
        operation="delete_script",
        op_kwargs={"ScriptId": script_id},
    )


async def describe(hub, ctx, script_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves properties for a Realtime script.  To request a script record, specify the script ID. If successful, an object containing the script properties is returned.  Learn more   Amazon GameLift Realtime Servers   Related actions   CreateScript | ListScripts | DescribeScript | UpdateScript | DeleteScript | All APIs by task

    Args:
        script_id(str): A unique identifier for the Realtime script to retrieve properties for. You can use either the script ID or ARN value.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.gamelift.script.describe(ctx, script_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.gamelift.script.describe script_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="gamelift",
        operation="describe_script",
        op_kwargs={"ScriptId": script_id},
    )


async def list_all(hub, ctx, limit: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves script records for all Realtime scripts that are associated with the AWS account in use.   Learn more   Amazon GameLift Realtime Servers   Related actions   CreateScript | ListScripts | DescribeScript | UpdateScript | DeleteScript | All APIs by task

    Args:
        limit(int, optional): The maximum number of results to return. Use this parameter with NextToken to get results as a set of sequential pages. Defaults to None.
        next_token(str, optional): A token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this operation. To start at the beginning of the result set, do not specify a value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.gamelift.script.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.gamelift.script.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="gamelift",
        operation="list_scripts",
        op_kwargs={"Limit": limit, "NextToken": next_token},
    )


async def update(
    hub,
    ctx,
    script_id: str,
    name: str = None,
    version: str = None,
    storage_location: Dict = None,
    zip_file: bytes = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates Realtime script metadata and content. To update script metadata, specify the script ID and provide updated name and/or version values.  To update script content, provide an updated zip file by pointing to either a local file or an Amazon S3 bucket location. You can use either method regardless of how the original script was uploaded. Use the Version parameter to track updates to the script. If the call is successful, the updated metadata is stored in the script record and a revised script is uploaded to the Amazon GameLift service. Once the script is updated and acquired by a fleet instance, the new version is used for all new game sessions.   Learn more   Amazon GameLift Realtime Servers   Related actions   CreateScript | ListScripts | DescribeScript | UpdateScript | DeleteScript | All APIs by task

    Args:
        script_id(str): A unique identifier for the Realtime script to update. You can use either the script ID or ARN value.
        name(str, optional): A descriptive label that is associated with a script. Script names do not need to be unique. Defaults to None.
        version(str, optional): Version information that is associated with a build or script. Version strings do not need to be unique. Defaults to None.
        storage_location(Dict, optional): The location of the Amazon S3 bucket where a zipped file containing your Realtime scripts is stored. The storage location must specify the Amazon S3 bucket name, the zip file name (the "key"), and a role ARN that allows Amazon GameLift to access the Amazon S3 storage location. The S3 bucket must be in the same Region where you want to create a new script. By default, Amazon GameLift uploads the latest version of the zip file; if you have S3 object versioning turned on, you can use the ObjectVersion parameter to specify an earlier version. . Defaults to None.
        zip_file(bytes, optional): A data object containing your Realtime scripts and dependencies as a zip file. The zip file can have one or multiple files. Maximum size of a zip file is 5 MB. When using the AWS CLI tool to create a script, this parameter is set to the zip file name. It must be prepended with the string "fileb://" to indicate that the file data is a binary object. For example: --zip-file fileb://myRealtimeScript.zip. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.gamelift.script.update(ctx, script_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.gamelift.script.update script_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="gamelift",
        operation="update_script",
        op_kwargs={
            "ScriptId": script_id,
            "Name": name,
            "Version": version,
            "StorageLocation": storage_location,
            "ZipFile": zip_file,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(
    hub, ctx, name: str, instance_id: str, association_status: Dict
) -> Dict:
    r"""
    **Autogenerated function**

    Updates the status of the Amazon Web Services Systems Manager document (SSM document) associated with the specified instance.

    Args:
        name(str): The name of the SSM document.
        instance_id(str): The ID of the instance.
        association_status(Dict): The association status.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.association.status.update(
                ctx, name=value, instance_id=value, association_status=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.association.status.update name=value, instance_id=value, association_status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="update_association_status",
        op_kwargs={
            "Name": name,
            "InstanceId": instance_id,
            "AssociationStatus": association_status,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub, ctx, engagement_id: str, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the engagements to contact channels that occurred by engaging a contact.

    Args:
        engagement_id(str): The Amazon Resource Name (ARN) of the engagement.
        next_token(str, optional): The pagination token to continue to the next page of results. Defaults to None.
        max_results(int, optional): The maximum number of engagements to contact channels to list per page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm_contact.page.by.engagement.list(ctx, engagement_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm_contact.page.by.engagement.list engagement_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm-contacts",
        operation="list_pages_by_engagement",
        op_kwargs={
            "EngagementId": engagement_id,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__virtualname__ = "async"


async def invoke(hub, ctx, function_name: str, invoke_args: bytes) -> Dict:
    r"""
    **Autogenerated function**

     For asynchronous function invocation, use Invoke.  Invokes a function asynchronously.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - my-function.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:my-function.    Partial ARN - 123456789012:function:my-function.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        invoke_args(bytes): The JSON that you want to provide to your Lambda function as input.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.async_.invoke(
                ctx, function_name=value, invoke_args=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.async.invoke function_name=value, invoke_args=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="invoke_async",
        op_kwargs={"FunctionName": function_name, "InvokeArgs": invoke_args},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, name: str, function_config: Dict, function_code: bytes
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a CloudFront function. To create a function, you provide the function code and some configuration information about the function. The response contains an Amazon Resource Name (ARN) that uniquely identifies the function. When you create a function, it’s in the DEVELOPMENT stage. In this stage, you can test the function with TestFunction, and update it with UpdateFunction. When you’re ready to use your function with a CloudFront distribution, use PublishFunction to copy the function from the DEVELOPMENT stage to LIVE. When it’s live, you can attach the function to a distribution’s cache behavior, using the function’s ARN.

    Args:
        name(str): A name to identify the function.
        function_config(Dict): Configuration information about the function, including an optional comment and the function’s runtime.
        function_code(bytes): The function code. For more information about writing a CloudFront function, see Writing function code for CloudFront Functions in the Amazon CloudFront Developer Guide.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.create(
                ctx, name=value, function_config=value, function_code=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.create name=value, function_config=value, function_code=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="create_function",
        op_kwargs={
            "Name": name,
            "FunctionConfig": function_config,
            "FunctionCode": function_code,
        },
    )


async def delete(hub, ctx, name: str, if_match: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a CloudFront function. You cannot delete a function if it’s associated with a cache behavior. First, update your distributions to remove the function association from all cache behaviors, then delete the function. To delete a function, you must provide the function’s name and version (ETag value). To get these values, you can use ListFunctions and DescribeFunction.

    Args:
        name(str): The name of the function that you are deleting.
        if_match(str): The current version (ETag value) of the function that you are deleting, which you can get using DescribeFunction.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.delete(ctx, name=value, if_match=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.delete name=value, if_match=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="delete_function",
        op_kwargs={"Name": name, "IfMatch": if_match},
    )


async def describe(hub, ctx, name: str, stage: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets configuration information and metadata about a CloudFront function, but not the function’s code. To get a function’s code, use GetFunction. To get configuration information and metadata about a function, you must provide the function’s name and stage. To get these values, you can use ListFunctions.

    Args:
        name(str): The name of the function that you are getting information about.
        stage(str, optional): The function’s stage, either DEVELOPMENT or LIVE. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.describe(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.describe name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="describe_function",
        op_kwargs={"Name": name, "Stage": stage},
    )


async def get(hub, ctx, name: str, stage: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets the code of a CloudFront function. To get configuration information and metadata about a function, use DescribeFunction. To get a function’s code, you must provide the function’s name and stage. To get these values, you can use ListFunctions.

    Args:
        name(str): The name of the function whose code you are getting.
        stage(str, optional): The function’s stage, either DEVELOPMENT or LIVE. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.get(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.get name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="get_function",
        op_kwargs={"Name": name, "Stage": stage},
    )


async def list_all(
    hub, ctx, marker: str = None, max_items: str = None, stage: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Gets a list of all CloudFront functions in your account. You can optionally apply a filter to return only the functions that are in the specified stage, either DEVELOPMENT or LIVE. You can optionally specify the maximum number of items to receive in the response. If the total number of items in the list exceeds the maximum that you specify, or the default maximum, the response is paginated. To get the next page of items, send a subsequent request that specifies the NextMarker value from the current response as the Marker value in the subsequent request.

    Args:
        marker(str, optional): Use this field when paginating results to indicate where to begin in your list of functions. The response includes functions in the list that occur after the marker. To get the next page of the list, set this field’s value to the value of NextMarker from the current page’s response. Defaults to None.
        max_items(str, optional): The maximum number of functions that you want in the response. Defaults to None.
        stage(str, optional): An optional filter to return only the functions that are in the specified stage, either DEVELOPMENT or LIVE. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="list_functions",
        op_kwargs={"Marker": marker, "MaxItems": max_items, "Stage": stage},
    )


async def publish(hub, ctx, name: str, if_match: str) -> Dict:
    r"""
    **Autogenerated function**

    Publishes a CloudFront function by copying the function code from the DEVELOPMENT stage to LIVE. This automatically updates all cache behaviors that are using this function to use the newly published copy in the LIVE stage. When a function is published to the LIVE stage, you can attach the function to a distribution’s cache behavior, using the function’s Amazon Resource Name (ARN). To publish a function, you must provide the function’s name and version (ETag value). To get these values, you can use ListFunctions and DescribeFunction.

    Args:
        name(str): The name of the function that you are publishing.
        if_match(str): The current version (ETag value) of the function that you are publishing, which you can get using DescribeFunction.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.publish(ctx, name=value, if_match=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.publish name=value, if_match=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="publish_function",
        op_kwargs={"Name": name, "IfMatch": if_match},
    )


async def test(
    hub, ctx, name: str, if_match: str, event_object: bytes, stage: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Tests a CloudFront function. To test a function, you provide an event object that represents an HTTP request or response that your CloudFront distribution could receive in production. CloudFront runs the function, passing it the event object that you provided, and returns the function’s result (the modified event object) in the response. The response also contains function logs and error messages, if any exist. For more information about testing functions, see Testing functions in the Amazon CloudFront Developer Guide. To test a function, you provide the function’s name and version (ETag value) along with the event object. To get the function’s name and version, you can use ListFunctions and DescribeFunction.

    Args:
        name(str): The name of the function that you are testing.
        if_match(str): The current version (ETag value) of the function that you are testing, which you can get using DescribeFunction.
        stage(str, optional): The stage of the function that you are testing, either DEVELOPMENT or LIVE. Defaults to None.
        event_object(bytes): The event object to test the function with. For more information about the structure of the event object, see Testing functions in the Amazon CloudFront Developer Guide.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.test(
                ctx, name=value, if_match=value, event_object=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.test name=value, if_match=value, event_object=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="test_function",
        op_kwargs={
            "Name": name,
            "IfMatch": if_match,
            "Stage": stage,
            "EventObject": event_object,
        },
    )


async def update(
    hub, ctx, name: str, if_match: str, function_config: Dict, function_code: bytes
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a CloudFront function. You can update a function’s code or the comment that describes the function. You cannot update a function’s name. To update a function, you provide the function’s name and version (ETag value) along with the updated function code. To get the name and version, you can use ListFunctions and DescribeFunction.

    Args:
        name(str): The name of the function that you are updating.
        if_match(str): The current version (ETag value) of the function that you are updating, which you can get using DescribeFunction.
        function_config(Dict): Configuration information about the function.
        function_code(bytes): The function code. For more information about writing a CloudFront function, see Writing function code for CloudFront Functions in the Amazon CloudFront Developer Guide.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.function.update(
                ctx, name=value, if_match=value, function_config=value, function_code=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.function.update name=value, if_match=value, function_config=value, function_code=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="update_function",
        op_kwargs={
            "Name": name,
            "IfMatch": if_match,
            "FunctionConfig": function_config,
            "FunctionCode": function_code,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def tag(hub, ctx, resource_arn: str, tags: Dict = None) -> None:
    r"""
    **Autogenerated function**

    Adds tags to a Greengrass resource. Valid resources are 'Group', 'ConnectorDefinition', 'CoreDefinition', 'DeviceDefinition', 'FunctionDefinition', 'LoggerDefinition', 'SubscriptionDefinition', 'ResourceDefinition', and 'BulkDeployment'.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the resource.
        tags(Dict, optional): The key-value pair for the resource tag. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.greengrass.resource.init.tag(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.greengrass.resource.init.tag resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="greengrass",
        operation="tag_resource",
        op_kwargs={"ResourceArn": resource_arn, "tags": tags},
    )


async def untag(hub, ctx, resource_arn: str, tag_keys: List) -> None:
    r"""
    **Autogenerated function**

    Remove resource tags from a Greengrass Resource.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the resource.
        tag_keys(List): An array of tag keys to delete.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.greengrass.resource.init.untag(
                ctx, resource_arn=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.greengrass.resource.init.untag resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="greengrass",
        operation="untag_resource",
        op_kwargs={"ResourceArn": resource_arn, "TagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create_multiple(hub, ctx, finding_types: List = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates sample findings.

    Args:
        finding_types(List, optional): An array that lists one or more types of findings to include in the set of sample findings. Currently, the only supported value is Policy:IAMUser/S3BucketEncryptionDisabled. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.macie2.sample_finding.create_multiple(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.macie2.sample_finding.create_multiple
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="macie2",
        operation="create_sample_findings",
        op_kwargs={"findingTypes": finding_types},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(
    hub,
    ctx,
    service: str,
    filter_: Dict = None,
    configuration: Dict = None,
    page_size: int = None,
    next_page_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates recommendations that help you save cost by identifying idle and underutilized Amazon EC2 instances. Recommendations are generated to either downsize or terminate instances, along with providing savings detail and metrics. For details on calculation and function, see Optimizing Your Cost with Rightsizing Recommendations in the AWS Billing and Cost Management User Guide.

    Args:
        filter_(Dict, optional): Use Expression to filter by cost or by usage. There are two patterns:    Simple dimension values - You can set the dimension name and values for the filters that you plan to use. For example, you can filter for REGION==us-east-1 OR REGION==us-west-1. For GetRightsizingRecommendation, the Region is a full name (for example, REGION==US East (N. Virginia). The Expression example looks like:  { "Dimensions": { "Key": "REGION", "Values": [ "us-east-1", “us-west-1” ] } }  The list of dimension values are OR'd together to retrieve cost or usage data. You can create Expression and DimensionValues objects using either with* methods or set* methods in multiple lines.    Compound dimension values with logical operations - You can use multiple Expression types and the logical operators AND/OR/NOT to create a list of one or more Expression objects. This allows you to filter on more advanced options. For example, you can filter on ((REGION == us-east-1 OR REGION == us-west-1) OR (TAG.Type == Type1)) AND (USAGE_TYPE != DataTransfer). The Expression for that looks like this:  { "And": [ {"Or": [ {"Dimensions": { "Key": "REGION", "Values": [ "us-east-1", "us-west-1" ] }}, {"Tags": { "Key": "TagName", "Values": ["Value1"] } } ]}, {"Not": {"Dimensions": { "Key": "USAGE_TYPE", "Values": ["DataTransfer"] }}} ] }    Because each Expression can have only one operator, the service returns an error if more than one is specified. The following example shows an Expression object that creates an error.    { "And": [ ... ], "DimensionValues": { "Dimension": "USAGE_TYPE", "Values": [ "DataTransfer" ] } }      For the GetRightsizingRecommendation action, a combination of OR and NOT is not supported. OR is not supported between different dimensions, or dimensions and tags. NOT operators aren't supported. Dimensions are also limited to LINKED_ACCOUNT, REGION, or RIGHTSIZING_TYPE. For the GetReservationPurchaseRecommendation action, only NOT is supported. AND and OR are not supported. Dimensions are limited to LINKED_ACCOUNT. . Defaults to None.
        configuration(Dict, optional):  Enables you to customize recommendations across two attributes. You can choose to view recommendations for instances within the same instance families or across different instance families. You can also choose to view your estimated savings associated with recommendations with consideration of existing Savings Plans or RI benefits, or neither. . Defaults to None.
        service(str): The specific service that you want recommendations for. The only valid value for GetRightsizingRecommendation is "AmazonEC2".
        page_size(int, optional): The number of recommendations that you want returned in a single response object. Defaults to None.
        next_page_token(str, optional): The pagination token that indicates the next set of results that you want to retrieve. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ce.rightsizing_recommendation.get(ctx, service=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ce.rightsizing_recommendation.get service=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ce",
        operation="get_rightsizing_recommendation",
        op_kwargs={
            "Filter": filter_,
            "Configuration": configuration,
            "Service": service,
            "PageSize": page_size,
            "NextPageToken": next_page_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, template_name: str, voice_template_request: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Creates a message template for messages that are sent through the voice channel.

    Args:
        template_name(str): The name of the message template. A template name must start with an alphanumeric character and can contain a maximum of 128 characters. The characters can be alphanumeric characters, underscores (_), or hyphens (-). Template names are case sensitive.
        voice_template_request(Dict): Specifies the content and settings for a message template that can be used in messages that are sent through the voice channel.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.pinpoint.voice.template.create(
                ctx, template_name=value, voice_template_request=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.pinpoint.voice.template.create template_name=value, voice_template_request=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="pinpoint",
        operation="create_voice_template",
        op_kwargs={
            "TemplateName": template_name,
            "VoiceTemplateRequest": voice_template_request,
        },
    )


async def delete(hub, ctx, template_name: str, version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a message template for messages that were sent through the voice channel.

    Args:
        template_name(str): The name of the message template. A template name must start with an alphanumeric character and can contain a maximum of 128 characters. The characters can be alphanumeric characters, underscores (_), or hyphens (-). Template names are case sensitive.
        version(str, optional): The unique identifier for the version of the message template to update, retrieve information about, or delete. To retrieve identifiers and other information for all the versions of a template, use the Template Versions resource. If specified, this value must match the identifier for an existing template version. If specified for an update operation, this value must match the identifier for the latest existing version of the template. This restriction helps ensure that race conditions don't occur. If you don't specify a value for this parameter, Amazon Pinpoint does the following: For a get operation, retrieves information about the active version of the template. For an update operation, saves the updates to (overwrites) the latest existing version of the template, if the create-new-version parameter isn't used or is set to false. For a delete operation, deletes the template, including all versions of the template. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.pinpoint.voice.template.delete(ctx, template_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.pinpoint.voice.template.delete template_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="pinpoint",
        operation="delete_voice_template",
        op_kwargs={"TemplateName": template_name, "Version": version},
    )


async def get(hub, ctx, template_name: str, version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the content and settings of a message template for messages that are sent through the voice channel.

    Args:
        template_name(str): The name of the message template. A template name must start with an alphanumeric character and can contain a maximum of 128 characters. The characters can be alphanumeric characters, underscores (_), or hyphens (-). Template names are case sensitive.
        version(str, optional): The unique identifier for the version of the message template to update, retrieve information about, or delete. To retrieve identifiers and other information for all the versions of a template, use the Template Versions resource. If specified, this value must match the identifier for an existing template version. If specified for an update operation, this value must match the identifier for the latest existing version of the template. This restriction helps ensure that race conditions don't occur. If you don't specify a value for this parameter, Amazon Pinpoint does the following: For a get operation, retrieves information about the active version of the template. For an update operation, saves the updates to (overwrites) the latest existing version of the template, if the create-new-version parameter isn't used or is set to false. For a delete operation, deletes the template, including all versions of the template. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.pinpoint.voice.template.get(ctx, template_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.pinpoint.voice.template.get template_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="pinpoint",
        operation="get_voice_template",
        op_kwargs={"TemplateName": template_name, "Version": version},
    )


async def update(
    hub,
    ctx,
    template_name: str,
    voice_template_request: Dict,
    create_new_version: bool = None,
    version: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an existing message template for messages that are sent through the voice channel.

    Args:
        create_new_version(bool, optional): Specifies whether to save the updates as a new version of the message template. Valid values are: true, save the updates as a new version; and, false, save the updates to (overwrite) the latest existing version of the template. If you don't specify a value for this parameter, Amazon Pinpoint saves the updates to (overwrites) the latest existing version of the template. If you specify a value of true for this parameter, don't specify a value for the version parameter. Otherwise, an error will occur. Defaults to None.
        template_name(str): The name of the message template. A template name must start with an alphanumeric character and can contain a maximum of 128 characters. The characters can be alphanumeric characters, underscores (_), or hyphens (-). Template names are case sensitive.
        version(str, optional): The unique identifier for the version of the message template to update, retrieve information about, or delete. To retrieve identifiers and other information for all the versions of a template, use the Template Versions resource. If specified, this value must match the identifier for an existing template version. If specified for an update operation, this value must match the identifier for the latest existing version of the template. This restriction helps ensure that race conditions don't occur. If you don't specify a value for this parameter, Amazon Pinpoint does the following: For a get operation, retrieves information about the active version of the template. For an update operation, saves the updates to (overwrites) the latest existing version of the template, if the create-new-version parameter isn't used or is set to false. For a delete operation, deletes the template, including all versions of the template. Defaults to None.
        voice_template_request(Dict): Specifies the content and settings for a message template that can be used in messages that are sent through the voice channel.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.pinpoint.voice.template.update(
                ctx, template_name=value, voice_template_request=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.pinpoint.voice.template.update template_name=value, voice_template_request=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="pinpoint",
        operation="update_voice_template",
        op_kwargs={
            "CreateNewVersion": create_new_version,
            "TemplateName": template_name,
            "Version": version,
            "VoiceTemplateRequest": voice_template_request,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, job_name: str, user_pool_id: str, cloud_watch_logs_role_arn: str
) -> Dict:
    r"""
    **Autogenerated function**

    Creates the user import job.

    Args:
        job_name(str): The job name for the user import job.
        user_pool_id(str): The user pool ID for the user pool that the users are being imported into.
        cloud_watch_logs_role_arn(str): The role ARN for the Amazon CloudWatch Logging role for the user import job.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.import_job.create(
                ctx, job_name=value, user_pool_id=value, cloud_watch_logs_role_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.import_job.create job_name=value, user_pool_id=value, cloud_watch_logs_role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="create_user_import_job",
        op_kwargs={
            "JobName": job_name,
            "UserPoolId": user_pool_id,
            "CloudWatchLogsRoleArn": cloud_watch_logs_role_arn,
        },
    )


async def describe(hub, ctx, user_pool_id: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes the user import job.

    Args:
        user_pool_id(str): The user pool ID for the user pool that the users are being imported into.
        job_id(str): The job ID for the user import job.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.import_job.describe(
                ctx, user_pool_id=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.import_job.describe user_pool_id=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="describe_user_import_job",
        op_kwargs={"UserPoolId": user_pool_id, "JobId": job_id},
    )


async def list_all(
    hub, ctx, user_pool_id: str, max_results: int, pagination_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the user import jobs.

    Args:
        user_pool_id(str): The user pool ID for the user pool that the users are being imported into.
        max_results(int): The maximum number of import jobs you want the request to return.
        pagination_token(str, optional): An identifier that was returned from the previous call to ListUserImportJobs, which can be used to return the next set of import jobs in the list. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.import_job.list_all(
                ctx, user_pool_id=value, max_results=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.import_job.list_all user_pool_id=value, max_results=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="list_user_import_jobs",
        op_kwargs={
            "UserPoolId": user_pool_id,
            "MaxResults": max_results,
            "PaginationToken": pagination_token,
        },
    )


async def start(hub, ctx, user_pool_id: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Starts the user import.

    Args:
        user_pool_id(str): The user pool ID for the user pool that the users are being imported into.
        job_id(str): The job ID for the user import job.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.import_job.start(
                ctx, user_pool_id=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.import_job.start user_pool_id=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="start_user_import_job",
        op_kwargs={"UserPoolId": user_pool_id, "JobId": job_id},
    )


async def stop(hub, ctx, user_pool_id: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Stops the user import job.

    Args:
        user_pool_id(str): The user pool ID for the user pool that the users are being imported into.
        job_id(str): The job ID for the user import job.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.import_job.stop(
                ctx, user_pool_id=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.import_job.stop user_pool_id=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="stop_user_import_job",
        op_kwargs={"UserPoolId": user_pool_id, "JobId": job_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(
    hub, ctx, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Args:
        next_token(str, optional): Token used for pagination. Defaults to None.
        max_results(int, optional): Max results used for pagination. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.efs.account_preference.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.efs.account_preference.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="efs",
        operation="describe_account_preferences",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )


async def put_multiple(hub, ctx, resource_id_type: str) -> Dict:
    r"""
    **Autogenerated function**

    Args:
        resource_id_type(str): A preference indicating a choice to use 63bit/32bit IDs for all applicable resources.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.efs.account_preference.put_multiple(ctx, resource_id_type=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.efs.account_preference.put_multiple resource_id_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="efs",
        operation="put_account_preferences",
        op_kwargs={"ResourceIdType": resource_id_type},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(
    hub,
    ctx,
    feature_group_name: str,
    record_identifier_value_as_string: str,
    event_time: str,
) -> None:
    r"""
    **Autogenerated function**

    Deletes a Record from a FeatureGroup. A new record will show up in the OfflineStore when the DeleteRecord API is called. This record will have a value of True in the is_deleted column.

    Args:
        feature_group_name(str): The name of the feature group to delete the record from. .
        record_identifier_value_as_string(str): The value for the RecordIdentifier that uniquely identifies the record, in string format. .
        event_time(str): Timestamp indicating when the deletion event occurred. EventTime can be used to query data at a certain point in time.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker_featurestore_runtime.record.delete(
                ctx,
                feature_group_name=value,
                record_identifier_value_as_string=value,
                event_time=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker_featurestore_runtime.record.delete feature_group_name=value, record_identifier_value_as_string=value, event_time=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker-featurestore-runtime",
        operation="delete_record",
        op_kwargs={
            "FeatureGroupName": feature_group_name,
            "RecordIdentifierValueAsString": record_identifier_value_as_string,
            "EventTime": event_time,
        },
    )


async def get(
    hub,
    ctx,
    feature_group_name: str,
    record_identifier_value_as_string: str,
    feature_names: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Use for OnlineStore serving from a FeatureStore. Only the latest records stored in the OnlineStore can be retrieved. If no Record with RecordIdentifierValue is found, then an empty result is returned.

    Args:
        feature_group_name(str): The name of the feature group in which you want to put the records.
        record_identifier_value_as_string(str): The value that corresponds to RecordIdentifier type and uniquely identifies the record in the FeatureGroup. .
        feature_names(List, optional): List of names of Features to be retrieved. If not specified, the latest value for all the Features are returned. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker_featurestore_runtime.record.get(
                ctx, feature_group_name=value, record_identifier_value_as_string=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker_featurestore_runtime.record.get feature_group_name=value, record_identifier_value_as_string=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker-featurestore-runtime",
        operation="get_record",
        op_kwargs={
            "FeatureGroupName": feature_group_name,
            "RecordIdentifierValueAsString": record_identifier_value_as_string,
            "FeatureNames": feature_names,
        },
    )


async def put(hub, ctx, feature_group_name: str, record: List) -> None:
    r"""
    **Autogenerated function**

    Used for data ingestion into the FeatureStore. The PutRecord API writes to both the OnlineStore and OfflineStore. If the record is the latest record for the recordIdentifier, the record is written to both the OnlineStore and OfflineStore. If the record is a historic record, it is written only to the OfflineStore.

    Args:
        feature_group_name(str): The name of the feature group that you want to insert the record into.
        record(List): List of FeatureValues to be inserted. This will be a full over-write. If you only want to update few of the feature values, do the following:   Use GetRecord to retrieve the latest record.   Update the record returned from GetRecord.    Use PutRecord to update feature values.  .
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker_featurestore_runtime.record.put(
                ctx, feature_group_name=value, record=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker_featurestore_runtime.record.put feature_group_name=value, record=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker-featurestore-runtime",
        operation="put_record",
        op_kwargs={"FeatureGroupName": feature_group_name, "Record": record},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    application_id: str,
    name: str,
    description: str = None,
    monitors: List = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    For each application, you define one or more environments. An environment is a logical deployment group of AppConfig targets, such as applications in a Beta or Production environment. You can also define environments for application subcomponents such as the Web, Mobile and Back-end components for your application. You can configure Amazon CloudWatch alarms for each environment. The system monitors alarms during a configuration deployment. If an alarm is triggered, the system rolls back the configuration.

    Args:
        application_id(str): The application ID.
        name(str): A name for the environment.
        description(str, optional): A description of the environment. Defaults to None.
        monitors(List, optional): Amazon CloudWatch alarms to monitor during the deployment process. Defaults to None.
        tags(Dict, optional): Metadata to assign to the environment. Tags help organize and categorize your AppConfig resources. Each tag consists of a key and an optional value, both of which you define. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appconfig.environment.create(
                ctx, application_id=value, name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appconfig.environment.create application_id=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appconfig",
        operation="create_environment",
        op_kwargs={
            "ApplicationId": application_id,
            "Name": name,
            "Description": description,
            "Monitors": monitors,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, application_id: str, environment_id: str) -> None:
    r"""
    **Autogenerated function**

    Delete an environment. Deleting an environment does not delete a configuration from a host.

    Args:
        application_id(str): The application ID that includes the environment you want to delete.
        environment_id(str): The ID of the environment you want to delete.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appconfig.environment.delete(
                ctx, application_id=value, environment_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appconfig.environment.delete application_id=value, environment_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appconfig",
        operation="delete_environment",
        op_kwargs={"ApplicationId": application_id, "EnvironmentId": environment_id},
    )


async def get(hub, ctx, application_id: str, environment_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieve information about an environment. An environment is a logical deployment group of AppConfig applications, such as applications in a Production environment or in an EU_Region environment. Each configuration deployment targets an environment. You can enable one or more Amazon CloudWatch alarms for an environment. If an alarm is triggered during a deployment, AppConfig roles back the configuration.

    Args:
        application_id(str): The ID of the application that includes the environment you want to get.
        environment_id(str): The ID of the environment you wnat to get.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appconfig.environment.get(
                ctx, application_id=value, environment_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appconfig.environment.get application_id=value, environment_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appconfig",
        operation="get_environment",
        op_kwargs={"ApplicationId": application_id, "EnvironmentId": environment_id},
    )


async def list_all(
    hub, ctx, application_id: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    List the environments for an application.

    Args:
        application_id(str): The application ID.
        max_results(int, optional): The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. Defaults to None.
        next_token(str, optional): A token to start the list. Use this token to get the next set of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appconfig.environment.list_all(ctx, application_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appconfig.environment.list_all application_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appconfig",
        operation="list_environments",
        op_kwargs={
            "ApplicationId": application_id,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )


async def update(
    hub,
    ctx,
    application_id: str,
    environment_id: str,
    name: str = None,
    description: str = None,
    monitors: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an environment.

    Args:
        application_id(str): The application ID.
        environment_id(str): The environment ID.
        name(str, optional): The name of the environment. Defaults to None.
        description(str, optional): A description of the environment. Defaults to None.
        monitors(List, optional): Amazon CloudWatch alarms to monitor during the deployment process. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appconfig.environment.update(
                ctx, application_id=value, environment_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appconfig.environment.update application_id=value, environment_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appconfig",
        operation="update_environment",
        op_kwargs={
            "ApplicationId": application_id,
            "EnvironmentId": environment_id,
            "Name": name,
            "Description": description,
            "Monitors": monitors,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


__virtualname__ = "data_collection_by_agent_id"


async def start(hub, ctx, agent_ids: List) -> Dict:
    r"""
    **Autogenerated function**

    Instructs the specified agents or connectors to start collecting data.

    Args:
        agent_ids(List): The IDs of the agents or connectors from which to start collecting data. If you send a request to an agent/connector ID that you do not have permission to contact, according to your AWS account, the service does not throw an exception. Instead, it returns the error in the Description field. If you send a request to multiple agents/connectors and you do not have permission to contact some of those agents/connectors, the system does not throw an exception. Instead, the system shows Failed in the Description field.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.discovery.data_collection_by_agent_id_.start(
                ctx, agent_ids=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.discovery.data_collection_by_agent_id.start agent_ids=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="discovery",
        operation="start_data_collection_by_agent_ids",
        op_kwargs={"agentIds": agent_ids},
    )


async def stop(hub, ctx, agent_ids: List) -> Dict:
    r"""
    **Autogenerated function**

    Instructs the specified agents or connectors to stop collecting data.

    Args:
        agent_ids(List): The IDs of the agents or connectors from which to stop collecting data.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.discovery.data_collection_by_agent_id_.stop(
                ctx, agent_ids=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.discovery.data_collection_by_agent_id.stop agent_ids=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="discovery",
        operation="stop_data_collection_by_agent_ids",
        op_kwargs={"agentIds": agent_ids},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, analyzer_arn: str, resource_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves information about a resource that was analyzed.

    Args:
        analyzer_arn(str): The ARN of the analyzer to retrieve information from.
        resource_arn(str): The ARN of the resource to retrieve information about.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.accessanalyzer.analyzed_resource.get(
                ctx, analyzer_arn=value, resource_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.accessanalyzer.analyzed_resource.get analyzer_arn=value, resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="accessanalyzer",
        operation="get_analyzed_resource",
        op_kwargs={"analyzerArn": analyzer_arn, "resourceArn": resource_arn},
    )


async def list_all(
    hub,
    ctx,
    analyzer_arn: str,
    max_results: int = None,
    next_token: str = None,
    resource_type: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves a list of resources of the specified type that have been analyzed by the specified analyzer..

    Args:
        analyzer_arn(str): The ARN of the analyzer to retrieve a list of analyzed resources from.
        max_results(int, optional): The maximum number of results to return in the response. Defaults to None.
        next_token(str, optional): A token used for pagination of results returned. Defaults to None.
        resource_type(str, optional): The type of resource. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.accessanalyzer.analyzed_resource.list_all(
                ctx, analyzer_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.accessanalyzer.analyzed_resource.list_all analyzer_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="accessanalyzer",
        operation="list_analyzed_resources",
        op_kwargs={
            "analyzerArn": analyzer_arn,
            "maxResults": max_results,
            "nextToken": next_token,
            "resourceType": resource_type,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import List


async def put(
    hub,
    ctx,
    alarm_name: str,
    evaluation_periods: int,
    comparison_operator: str,
    alarm_description: str = None,
    actions_enabled: bool = None,
    ok_actions: List = None,
    alarm_actions: List = None,
    insufficient_data_actions: List = None,
    metric_name: str = None,
    namespace: str = None,
    statistic: str = None,
    extended_statistic: str = None,
    dimensions: List = None,
    period: int = None,
    unit: str = None,
    datapoints_to_alarm: int = None,
    threshold: float = None,
    treat_missing_data: str = None,
    evaluate_low_sample_count_percentile: str = None,
    metrics: List = None,
    tags: List = None,
    threshold_metric_id: str = None,
) -> None:
    r"""
    **Autogenerated function**

    Creates or updates an alarm and associates it with the specified metric, metric math expression, or anomaly detection model. Alarms based on anomaly detection models cannot have Auto Scaling actions. When this operation creates an alarm, the alarm state is immediately set to INSUFFICIENT_DATA. The alarm is then evaluated and its state is set appropriately. Any actions associated with the new state are then executed. When you update an existing alarm, its state is left unchanged, but the update completely overwrites the previous configuration of the alarm. If you are an IAM user, you must have Amazon EC2 permissions for some alarm operations:   The iam:CreateServiceLinkedRole for all alarms with EC2 actions   The iam:CreateServiceLinkedRole to create an alarm with Systems Manager OpsItem actions.   The first time you create an alarm in the Management Console, the CLI, or by using the PutMetricAlarm API, CloudWatch creates the necessary service-linked role for you. The service-linked roles are called AWSServiceRoleForCloudWatchEvents and AWSServiceRoleForCloudWatchAlarms_ActionSSM. For more information, see Amazon Web Services service-linked role.  Cross-account alarms  You can set an alarm on metrics in the current account, or in another account. To create a cross-account alarm that watches a metric in a different account, you must have completed the following pre-requisites:   The account where the metrics are located (the sharing account) must already have a sharing role named CloudWatch-CrossAccountSharingRole. If it does not already have this role, you must create it using the instructions in Set up a sharing account in  Cross-account cross-Region CloudWatch console. The policy for that role must grant access to the ID of the account where you are creating the alarm.    The account where you are creating the alarm (the monitoring account) must already have a service-linked role named AWSServiceRoleForCloudWatchCrossAccount to allow CloudWatch to assume the sharing role in the sharing account. If it does not, you must create it following the directions in Set up a monitoring account in  Cross-account cross-Region CloudWatch console.

    Args:
        alarm_name(str): The name for the alarm. This name must be unique within the Region.
        alarm_description(str, optional): The description for the alarm. Defaults to None.
        actions_enabled(bool, optional): Indicates whether actions should be executed during any changes to the alarm state. The default is TRUE. Defaults to None.
        ok_actions(List, optional): The actions to execute when this alarm transitions to an OK state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: arn:aws:automate:region:ec2:stop | arn:aws:automate:region:ec2:terminate | arn:aws:automate:region:ec2:recover | arn:aws:automate:region:ec2:reboot | arn:aws:sns:region:account-id:sns-topic-name  | arn:aws:autoscaling:region:account-id:scalingPolicy:policy-id:autoScalingGroupName/group-friendly-name:policyName/policy-friendly-name   Valid Values (for use with IAM roles): arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Stop/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Terminate/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Reboot/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Recover/1.0 . Defaults to None.
        alarm_actions(List, optional): The actions to execute when this alarm transitions to the ALARM state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: arn:aws:automate:region:ec2:stop | arn:aws:automate:region:ec2:terminate | arn:aws:automate:region:ec2:recover | arn:aws:automate:region:ec2:reboot | arn:aws:sns:region:account-id:sns-topic-name  | arn:aws:autoscaling:region:account-id:scalingPolicy:policy-id:autoScalingGroupName/group-friendly-name:policyName/policy-friendly-name  | arn:aws:ssm:region:account-id:opsitem:severity  | arn:aws:ssm-incidents::account-id:response-plan:response-plan-name   Valid Values (for use with IAM roles): arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Stop/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Terminate/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Reboot/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Recover/1.0 . Defaults to None.
        insufficient_data_actions(List, optional): The actions to execute when this alarm transitions to the INSUFFICIENT_DATA state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: arn:aws:automate:region:ec2:stop | arn:aws:automate:region:ec2:terminate | arn:aws:automate:region:ec2:recover | arn:aws:automate:region:ec2:reboot | arn:aws:sns:region:account-id:sns-topic-name  | arn:aws:autoscaling:region:account-id:scalingPolicy:policy-id:autoScalingGroupName/group-friendly-name:policyName/policy-friendly-name   Valid Values (for use with IAM roles): >arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Stop/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Terminate/1.0 | arn:aws:swf:region:account-id:action/actions/AWS_EC2.InstanceId.Reboot/1.0 . Defaults to None.
        metric_name(str, optional): The name for the metric associated with the alarm. For each PutMetricAlarm operation, you must specify either MetricName or a Metrics array. If you are creating an alarm based on a math expression, you cannot specify this parameter, or any of the Dimensions, Period, Namespace, Statistic, or ExtendedStatistic parameters. Instead, you specify all this information in the Metrics array. Defaults to None.
        namespace(str, optional): The namespace for the metric associated specified in MetricName. Defaults to None.
        statistic(str, optional): The statistic for the metric specified in MetricName, other than percentile. For percentile statistics, use ExtendedStatistic. When you call PutMetricAlarm and specify a MetricName, you must specify either Statistic or ExtendedStatistic, but not both. Defaults to None.
        extended_statistic(str, optional): The percentile statistic for the metric specified in MetricName. Specify a value between p0.0 and p100. When you call PutMetricAlarm and specify a MetricName, you must specify either Statistic or ExtendedStatistic, but not both. Defaults to None.
        dimensions(List, optional): The dimensions for the metric specified in MetricName. Defaults to None.
        period(int, optional): The length, in seconds, used each time the metric specified in MetricName is evaluated. Valid values are 10, 30, and any multiple of 60.  Period is required for alarms based on static thresholds. If you are creating an alarm based on a metric math expression, you specify the period for each metric within the objects in the Metrics array. Be sure to specify 10 or 30 only for metrics that are stored by a PutMetricData call with a StorageResolution of 1. If you specify a period of 10 or 30 for a metric that does not have sub-minute resolution, the alarm still attempts to gather data at the period rate that you specify. In this case, it does not receive data for the attempts that do not correspond to a one-minute data resolution, and the alarm might often lapse into INSUFFICENT_DATA status. Specifying 10 or 30 also sets this alarm as a high-resolution alarm, which has a higher charge than other alarms. For more information about pricing, see Amazon CloudWatch Pricing. An alarm's total current evaluation period can be no longer than one day, so Period multiplied by EvaluationPeriods cannot be more than 86,400 seconds. Defaults to None.
        unit(str, optional): The unit of measure for the statistic. For example, the units for the Amazon EC2 NetworkIn metric are Bytes because NetworkIn tracks the number of bytes that an instance receives on all network interfaces. You can also specify a unit when you create a custom metric. Units help provide conceptual meaning to your data. Metric data points that specify a unit of measure, such as Percent, are aggregated separately. If you don't specify Unit, CloudWatch retrieves all unit types that have been published for the metric and attempts to evaluate the alarm. Usually, metrics are published with only one unit, so the alarm works as intended. However, if the metric is published with multiple types of units and you don't specify a unit, the alarm's behavior is not defined and it behaves predictably. We recommend omitting Unit so that you don't inadvertently specify an incorrect unit that is not published for this metric. Doing so causes the alarm to be stuck in the INSUFFICIENT DATA state. Defaults to None.
        evaluation_periods(int): The number of periods over which data is compared to the specified threshold. If you are setting an alarm that requires that a number of consecutive data points be breaching to trigger the alarm, this value specifies that number. If you are setting an "M out of N" alarm, this value is the N. An alarm's total current evaluation period can be no longer than one day, so this number multiplied by Period cannot be more than 86,400 seconds.
        datapoints_to_alarm(int, optional): The number of data points that must be breaching to trigger the alarm. This is used only if you are setting an "M out of N" alarm. In that case, this value is the M. For more information, see Evaluating an Alarm in the Amazon CloudWatch User Guide. Defaults to None.
        threshold(float, optional): The value against which the specified statistic is compared. This parameter is required for alarms based on static thresholds, but should not be used for alarms based on anomaly detection models. Defaults to None.
        comparison_operator(str):  The arithmetic operation to use when comparing the specified statistic and threshold. The specified statistic value is used as the first operand. The values LessThanLowerOrGreaterThanUpperThreshold, LessThanLowerThreshold, and GreaterThanUpperThreshold are used only for alarms based on anomaly detection models.
        treat_missing_data(str, optional):  Sets how this alarm is to handle missing data points. If TreatMissingData is omitted, the default behavior of missing is used. For more information, see Configuring How CloudWatch Alarms Treats Missing Data. Valid Values: breaching | notBreaching | ignore | missing . Defaults to None.
        evaluate_low_sample_count_percentile(str, optional):  Used only for alarms based on percentiles. If you specify ignore, the alarm state does not change during periods with too few data points to be statistically significant. If you specify evaluate or omit this parameter, the alarm is always evaluated and possibly changes state no matter how many data points are available. For more information, see Percentile-Based CloudWatch Alarms and Low Data Samples. Valid Values: evaluate | ignore . Defaults to None.
        metrics(List, optional): An array of MetricDataQuery structures that enable you to create an alarm based on the result of a metric math expression. For each PutMetricAlarm operation, you must specify either MetricName or a Metrics array. Each item in the Metrics array either retrieves a metric or performs a math expression. One item in the Metrics array is the expression that the alarm watches. You designate this expression by setting ReturnData to true for this object in the array. For more information, see MetricDataQuery. If you use the Metrics parameter, you cannot include the MetricName, Dimensions, Period, Namespace, Statistic, or ExtendedStatistic parameters of PutMetricAlarm in the same operation. Instead, you retrieve the metrics you are using in your math expression as part of the Metrics array. Defaults to None.
        tags(List, optional): A list of key-value pairs to associate with the alarm. You can associate as many as 50 tags with an alarm. Tags can help you organize and categorize your resources. You can also use them to scope user permissions by granting a user permission to access or change only resources with certain tag values. If you are using this operation to update an existing alarm, any tags you specify in this parameter are ignored. To change the tags of an existing alarm, use TagResource or UntagResource. Defaults to None.
        threshold_metric_id(str, optional): If this is an alarm based on an anomaly detection model, make this value match the ID of the ANOMALY_DETECTION_BAND function. For an example of how to use this parameter, see the Anomaly Detection Model Alarm example on this page. If your alarm uses this parameter, it cannot have Auto Scaling actions. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudwatch.metric.alarm.put(
                ctx, alarm_name=value, evaluation_periods=value, comparison_operator=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudwatch.metric.alarm.put alarm_name=value, evaluation_periods=value, comparison_operator=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudwatch",
        operation="put_metric_alarm",
        op_kwargs={
            "AlarmName": alarm_name,
            "AlarmDescription": alarm_description,
            "ActionsEnabled": actions_enabled,
            "OKActions": ok_actions,
            "AlarmActions": alarm_actions,
            "InsufficientDataActions": insufficient_data_actions,
            "MetricName": metric_name,
            "Namespace": namespace,
            "Statistic": statistic,
            "ExtendedStatistic": extended_statistic,
            "Dimensions": dimensions,
            "Period": period,
            "Unit": unit,
            "EvaluationPeriods": evaluation_periods,
            "DatapointsToAlarm": datapoints_to_alarm,
            "Threshold": threshold,
            "ComparisonOperator": comparison_operator,
            "TreatMissingData": treat_missing_data,
            "EvaluateLowSampleCountPercentile": evaluate_low_sample_count_percentile,
            "Metrics": metrics,
            "Tags": tags,
            "ThresholdMetricId": threshold_metric_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def analyze(
    hub, ctx, document: Dict, feature_types: List, human_loop_config: Dict = None
) -> Dict:
    r"""
    **Autogenerated function**

    Analyzes an input document for relationships between detected items.  The types of information returned are as follows:    Form data (key-value pairs). The related information is returned in two Block objects, each of type KEY_VALUE_SET: a KEY Block object and a VALUE Block object. For example, Name: Ana Silva Carolina contains a key and value. Name: is the key. Ana Silva Carolina is the value.   Table and table cell data. A TABLE Block object contains information about a detected table. A CELL Block object is returned for each cell in a table.   Lines and words of text. A LINE Block object contains one or more WORD Block objects. All lines and words that are detected in the document are returned (including text that doesn't have a relationship with the value of FeatureTypes).    Selection elements such as check boxes and option buttons (radio buttons) can be detected in form data and in tables. A SELECTION_ELEMENT Block object contains information about a selection element, including the selection status. You can choose which type of analysis to perform by specifying the FeatureTypes list.  The output is returned in a list of Block objects.  AnalyzeDocument is a synchronous operation. To analyze documents asynchronously, use StartDocumentAnalysis. For more information, see Document Text Analysis.

    Args:
        document(Dict): The input document as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI to call Amazon Textract operations, you can't pass image bytes. The document must be an image in JPEG or PNG format. If you're using an AWS SDK to call Amazon Textract, you might not need to base64-encode image bytes that are passed using the Bytes field. .
        feature_types(List): A list of the types of analysis to perform. Add TABLES to the list to return information about the tables that are detected in the input document. Add FORMS to return detected form data. To perform both types of analysis, add TABLES and FORMS to FeatureTypes. All lines and words detected in the document are included in the response (including text that isn't related to the value of FeatureTypes). .
        human_loop_config(Dict, optional): Sets the configuration for the human in the loop workflow for analyzing documents. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.textract.document.init.analyze(
                ctx, document=value, feature_types=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.textract.document.init.analyze document=value, feature_types=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="textract",
        operation="analyze_document",
        op_kwargs={
            "Document": document,
            "FeatureTypes": feature_types,
            "HumanLoopConfig": human_loop_config,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(hub, ctx, routing_control_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Get the state for a routing control. A routing control is a simple on/off switch that you can use to route traffic to cells. When the state is On, traffic flows to a cell. When it's off, traffic does not flow.  Before you can create a routing control, you first must create a cluster to host the control. For more information, see CreateCluster. Access one of the endpoints for the cluster to get or update the routing control state to redirect traffic. For more information about working with routing controls, see Routing control in the Route 53 Application Recovery Controller Developer Guide.

    Args:
        routing_control_arn(str): The Amazon Resource Number (ARN) for the routing control that you want to get the state for.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53_recovery_cluster_routing_control_state.get(
                ctx, routing_control_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53_recovery_cluster_routing_control_state.get routing_control_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="route53-recovery-cluster",
        operation="get_routing_control_state",
        op_kwargs={"RoutingControlArn": routing_control_arn},
    )


async def update(
    hub, ctx, routing_control_arn: str, routing_control_state: str
) -> Dict:
    r"""
    **Autogenerated function**

    Set the state of the routing control to reroute traffic. You can set the value to be On or Off. When the state is On, traffic flows to a cell. When it's off, traffic does not flow. For more information about working with routing controls, see Routing control in the Route 53 Application Recovery Controller Developer Guide.

    Args:
        routing_control_arn(str): The Amazon Resource Number (ARN) for the routing control that you want to update the state for.
        routing_control_state(str): The state of the routing control. You can set the value to be On or Off.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53_recovery_cluster_routing_control_state.update(
                ctx, routing_control_arn=value, routing_control_state=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53_recovery_cluster_routing_control_state.update routing_control_arn=value, routing_control_state=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="route53-recovery-cluster",
        operation="update_routing_control_state",
        op_kwargs={
            "RoutingControlArn": routing_control_arn,
            "RoutingControlState": routing_control_state,
        },
    )


async def update_multiple(hub, ctx, update_routing_control_state_entries: List) -> Dict:
    r"""
    **Autogenerated function**

    Set multiple routing control states. You can set the value for each state to be On or Off. When the state is On, traffic flows to a cell. When it's off, traffic does not flow. For more information about working with routing controls, see Routing control in the Route 53 Application Recovery Controller Developer Guide.

    Args:
        update_routing_control_state_entries(List): A set of routing control entries that you want to update.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53_recovery_cluster_routing_control_state.update_multiple(
                ctx, update_routing_control_state_entries=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53_recovery_cluster_routing_control_state.update_multiple update_routing_control_state_entries=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="route53-recovery-cluster",
        operation="update_routing_control_states",
        op_kwargs={
            "UpdateRoutingControlStateEntries": update_routing_control_state_entries
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def register(
    hub,
    ctx,
    window_id: str,
    task_arn: str,
    task_type: str,
    targets: List = None,
    service_role_arn: str = None,
    task_parameters: Dict = None,
    task_invocation_parameters: Dict = None,
    priority: int = None,
    max_concurrency: str = None,
    max_errors: str = None,
    logging_info: Dict = None,
    name: str = None,
    description: str = None,
    client_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Adds a new task to a maintenance window.

    Args:
        window_id(str): The ID of the maintenance window the task should be added to.
        targets(List, optional): The targets (either instances or maintenance window targets).  One or more targets must be specified for maintenance window Run Command-type tasks. Depending on the task, targets are optional for other maintenance window task types (Automation, Lambda, and Step Functions). For more information about running tasks that don't specify targets, see Registering maintenance window tasks without targets in the Amazon Web Services Systems Manager User Guide.  Specify instances using the following format:   Key=InstanceIds,Values=<instance-id-1>,<instance-id-2>  Specify maintenance window targets using the following format:  Key=WindowTargetIds,Values=<window-target-id-1>,<window-target-id-2> . Defaults to None.
        task_arn(str): The ARN of the task to run.
        service_role_arn(str, optional): The Amazon Resource Name (ARN) of the IAM service role for Amazon Web Services Systems Manager to assume when running a maintenance window task. If you do not specify a service role ARN, Systems Manager uses your account's service-linked role. If no service-linked role for Systems Manager exists in your account, it is created when you run RegisterTaskWithMaintenanceWindow. For more information, see the following topics in the in the Amazon Web Services Systems Manager User Guide:    Using service-linked roles for Systems Manager     Should I use a service-linked role or a custom service role to run maintenance window tasks?    . Defaults to None.
        task_type(str): The type of task being registered.
        task_parameters(Dict, optional): The parameters that should be passed to the task when it is run.   TaskParameters has been deprecated. To specify parameters to pass to a task when it runs, instead use the Parameters option in the TaskInvocationParameters structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see MaintenanceWindowTaskInvocationParameters. . Defaults to None.
        task_invocation_parameters(Dict, optional): The parameters that the task should use during execution. Populate only the fields that match the task type. All other fields should be empty. . Defaults to None.
        priority(int, optional): The priority of the task in the maintenance window, the lower the number the higher the priority. Tasks in a maintenance window are scheduled in priority order with tasks that have the same priority scheduled in parallel. Defaults to None.
        max_concurrency(str, optional): The maximum number of targets this task can be run for in parallel.  For maintenance window tasks without a target specified, you can't supply a value for this option. Instead, the system inserts a placeholder value of 1. This value doesn't affect the running of your task. . Defaults to None.
        max_errors(str, optional): The maximum number of errors allowed before this task stops being scheduled.  For maintenance window tasks without a target specified, you can't supply a value for this option. Instead, the system inserts a placeholder value of 1. This value doesn't affect the running of your task. . Defaults to None.
        logging_info(Dict, optional): A structure containing information about an Amazon Simple Storage Service (Amazon S3) bucket to write instance-level logs to.    LoggingInfo has been deprecated. To specify an Amazon Simple Storage Service (Amazon S3) bucket to contain logs, instead use the OutputS3BucketName and OutputS3KeyPrefix options in the TaskInvocationParameters structure. For information about how Amazon Web Services Systems Manager handles these options for the supported maintenance window task types, see MaintenanceWindowTaskInvocationParameters. . Defaults to None.
        name(str, optional): An optional name for the task. Defaults to None.
        description(str, optional): An optional description for the task. Defaults to None.
        client_token(str, optional): User-provided idempotency token. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.task.with_maintenance_window.register(
                ctx, window_id=value, task_arn=value, task_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.task.with_maintenance_window.register window_id=value, task_arn=value, task_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="register_task_with_maintenance_window",
        op_kwargs={
            "WindowId": window_id,
            "Targets": targets,
            "TaskArn": task_arn,
            "ServiceRoleArn": service_role_arn,
            "TaskType": task_type,
            "TaskParameters": task_parameters,
            "TaskInvocationParameters": task_invocation_parameters,
            "Priority": priority,
            "MaxConcurrency": max_concurrency,
            "MaxErrors": max_errors,
            "LoggingInfo": logging_info,
            "Name": name,
            "Description": description,
            "ClientToken": client_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def export(
    hub,
    ctx,
    export_attribute_name: str,
    server_name: str,
    input_attributes: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Exports a specified server engine attribute as a base64-encoded string. For example, you can export user data that you can use in EC2 to associate nodes with a server.   This operation is synchronous.   A ValidationException is raised when parameters of the request are not valid. A ResourceNotFoundException is thrown when the server does not exist. An InvalidStateException is thrown when the server is in any of the following states: CREATING, TERMINATED, FAILED or DELETING.

    Args:
        export_attribute_name(str): The name of the export attribute. Currently, the supported export attribute is Userdata. This exports a user data script that includes parameters and values provided in the InputAttributes list.
        server_name(str): The name of the server from which you are exporting the attribute.
        input_attributes(List, optional): The list of engine attributes. The list type is EngineAttribute. An EngineAttribute list item is a pair that includes an attribute name and its value. For the Userdata ExportAttributeName, the following are supported engine attribute names.    RunList In Chef, a list of roles or recipes that are run in the specified order. In Puppet, this parameter is ignored.    OrganizationName In Chef, an organization name. AWS OpsWorks for Chef Automate always creates the organization default. In Puppet, this parameter is ignored.    NodeEnvironment In Chef, a node environment (for example, development, staging, or one-box). In Puppet, this parameter is ignored.    NodeClientVersion In Chef, the version of the Chef engine (three numbers separated by dots, such as 13.8.5). If this attribute is empty, OpsWorks for Chef Automate uses the most current version. In Puppet, this parameter is ignored.  . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.engine_attribute.export(
                ctx, export_attribute_name=value, server_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.engine_attribute.export export_attribute_name=value, server_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="export_server_engine_attribute",
        op_kwargs={
            "ExportAttributeName": export_attribute_name,
            "ServerName": server_name,
            "InputAttributes": input_attributes,
        },
    )


async def update_multiple(
    hub, ctx, server_name: str, attribute_name: str, attribute_value: str = None
) -> Dict:
    r"""
    **Autogenerated function**

     Updates engine-specific attributes on a specified server. The server enters the MODIFYING state when this operation is in progress. Only one update can occur at a time. You can use this command to reset a Chef server's public key (CHEF_PIVOTAL_KEY) or a Puppet server's admin password (PUPPET_ADMIN_PASSWORD).   This operation is asynchronous.   This operation can only be called for servers in HEALTHY or UNHEALTHY states. Otherwise, an InvalidStateException is raised. A ResourceNotFoundException is thrown when the server does not exist. A ValidationException is raised when parameters of the request are not valid.

    Args:
        server_name(str): The name of the server to update. .
        attribute_name(str): The name of the engine attribute to update. .
        attribute_value(str, optional): The value to set for the attribute. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.engine_attribute.update_multiple(
                ctx, server_name=value, attribute_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.engine_attribute.update_multiple server_name=value, attribute_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="update_server_engine_attributes",
        op_kwargs={
            "ServerName": server_name,
            "AttributeName": attribute_name,
            "AttributeValue": attribute_value,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def put_multiple(
    hub,
    ctx,
    progress_update_stream: str,
    migration_task_name: str,
    resource_attribute_list: List,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Provides identifying details of the resource being migrated so that it can be associated in the Application Discovery Service repository. This association occurs asynchronously after PutResourceAttributes returns.    Keep in mind that subsequent calls to PutResourceAttributes will override previously stored attributes. For example, if it is first called with a MAC address, but later, it is desired to add an IP address, it will then be required to call it with both the IP and MAC addresses to prevent overriding the MAC address.   Note the instructions regarding the special use case of the  ResourceAttributeList  parameter when specifying any "VM" related value.     Because this is an asynchronous call, it will always return 200, whether an association occurs or not. To confirm if an association was found based on the provided details, call ListDiscoveredResources.

    Args:
        progress_update_stream(str): The name of the ProgressUpdateStream. .
        migration_task_name(str): Unique identifier that references the migration task. Do not store personal data in this field. .
        resource_attribute_list(List): Information about the resource that is being migrated. This data will be used to map the task to a resource in the Application Discovery Service repository.  Takes the object array of ResourceAttribute where the Type field is reserved for the following values: IPV4_ADDRESS | IPV6_ADDRESS | MAC_ADDRESS | FQDN | VM_MANAGER_ID | VM_MANAGED_OBJECT_REFERENCE | VM_NAME | VM_PATH | BIOS_ID | MOTHERBOARD_SERIAL_NUMBER where the identifying value can be a string up to 256 characters.     If any "VM" related value is set for a ResourceAttribute object, it is required that VM_MANAGER_ID, as a minimum, is always set. If VM_MANAGER_ID is not set, then all "VM" fields will be discarded and "VM" fields will not be used for matching the migration task to a server in Application Discovery Service repository. See the Example section below for a use case of specifying "VM" related values.    If a server you are trying to match has multiple IP or MAC addresses, you should provide as many as you know in separate type/value pairs passed to the ResourceAttributeList parameter to maximize the chances of matching.   .
        dry_run(bool, optional): Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mgh.resource_attribute.put_multiple(
                ctx,
                progress_update_stream=value,
                migration_task_name=value,
                resource_attribute_list=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mgh.resource_attribute.put_multiple progress_update_stream=value, migration_task_name=value, resource_attribute_list=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mgh",
        operation="put_resource_attributes",
        op_kwargs={
            "ProgressUpdateStream": progress_update_stream,
            "MigrationTaskName": migration_task_name,
            "ResourceAttributeList": resource_attribute_list,
            "DryRun": dry_run,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the current status of a change batch request. The status is one of the following values:    PENDING indicates that the changes in this request have not propagated to all Amazon Route 53 DNS servers. This is the initial status of all change batch requests.    INSYNC indicates that the changes have propagated to all Route 53 DNS servers.

    Args:
        id_(str): The ID of the change batch request. The value that you specify here is the value that ChangeResourceRecordSets returned in the Id element when you submitted the request.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53.change.get(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53.change.get id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="route53", operation="get_change", op_kwargs={"Id": id_}
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def start(
    hub,
    ctx,
    resource_arn: str,
    mode: str,
    kms_key_id: str,
    apply_immediately: bool = None,
    engine_native_audit_fields_included: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Starts a database activity stream to monitor activity on the database. For more information, see Database Activity Streams in the Amazon Aurora User Guide.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the DB cluster, for example, arn:aws:rds:us-east-1:12345667890:cluster:das-cluster.
        mode(str): Specifies the mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. .
        kms_key_id(str): The Amazon Web Services KMS key identifier for encrypting messages in the database activity stream. The Amazon Web Services KMS key identifier is the key ARN, key ID, alias ARN, or alias name for the Amazon Web Services KMS customer master key (CMK).
        apply_immediately(bool, optional): Specifies whether or not the database activity stream is to start as soon as possible, regardless of the maintenance window for the database. Defaults to None.
        engine_native_audit_fields_included(bool, optional): Specifies whether the database activity stream includes engine-native audit fields. This option only applies to an Oracle DB instance. By default, no engine-native audit fields are included. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rds.activity_stream.start(
                ctx, resource_arn=value, mode=value, kms_key_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rds.activity_stream.start resource_arn=value, mode=value, kms_key_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rds",
        operation="start_activity_stream",
        op_kwargs={
            "ResourceArn": resource_arn,
            "Mode": mode,
            "KmsKeyId": kms_key_id,
            "ApplyImmediately": apply_immediately,
            "EngineNativeAuditFieldsIncluded": engine_native_audit_fields_included,
        },
    )


async def stop(hub, ctx, resource_arn: str, apply_immediately: bool = None) -> Dict:
    r"""
    **Autogenerated function**

    Stops a database activity stream that was started using the Amazon Web Services console, the start-activity-stream CLI command, or the StartActivityStream action. For more information, see Database Activity Streams in the Amazon Aurora User Guide.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the DB cluster for the database activity stream. For example, arn:aws:rds:us-east-1:12345667890:cluster:das-cluster. .
        apply_immediately(bool, optional): Specifies whether or not the database activity stream is to stop as soon as possible, regardless of the maintenance window for the database. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rds.activity_stream.stop(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rds.activity_stream.stop resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rds",
        operation="stop_activity_stream",
        op_kwargs={"ResourceArn": resource_arn, "ApplyImmediately": apply_immediately},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(
    hub,
    ctx,
    metric_data_queries: List,
    start_time: str,
    end_time: str,
    next_token: str = None,
    scan_by: str = None,
    max_datapoints: int = None,
    label_options: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    You can use the GetMetricData API to retrieve as many as 500 different metrics in a single request, with a total of as many as 100,800 data points. You can also optionally perform math expressions on the values of the returned statistics, to create new time series that represent new insights into your data. For example, using Lambda metrics, you could divide the Errors metric by the Invocations metric to get an error rate time series. For more information about metric math expressions, see Metric Math Syntax and Functions in the Amazon CloudWatch User Guide. Calls to the GetMetricData API have a different pricing structure than calls to GetMetricStatistics. For more information about pricing, see Amazon CloudWatch Pricing. Amazon CloudWatch retains metric data as follows:   Data points with a period of less than 60 seconds are available for 3 hours. These data points are high-resolution metrics and are available only for custom metrics that have been defined with a StorageResolution of 1.   Data points with a period of 60 seconds (1-minute) are available for 15 days.   Data points with a period of 300 seconds (5-minute) are available for 63 days.   Data points with a period of 3600 seconds (1 hour) are available for 455 days (15 months).   Data points that are initially published with a shorter period are aggregated together for long-term storage. For example, if you collect data using a period of 1 minute, the data remains available for 15 days with 1-minute resolution. After 15 days, this data is still available, but is aggregated and retrievable only with a resolution of 5 minutes. After 63 days, the data is further aggregated and is available with a resolution of 1 hour. If you omit Unit in your request, all data that was collected with any unit is returned, along with the corresponding units that were specified when the data was reported to CloudWatch. If you specify a unit, the operation returns only data that was collected with that unit specified. If you specify a unit that does not match the data collected, the results of the operation are null. CloudWatch does not perform unit conversions.

    Args:
        metric_data_queries(List): The metric queries to be returned. A single GetMetricData call can include as many as 500 MetricDataQuery structures. Each of these structures can specify either a metric to retrieve, or a math expression to perform on retrieved data. .
        start_time(str): The time stamp indicating the earliest data to be returned. The value specified is inclusive; results include data points with the specified time stamp.  CloudWatch rounds the specified time stamp as follows:   Start time less than 15 days ago - Round down to the nearest whole minute. For example, 12:32:34 is rounded down to 12:32:00.   Start time between 15 and 63 days ago - Round down to the nearest 5-minute clock interval. For example, 12:32:34 is rounded down to 12:30:00.   Start time greater than 63 days ago - Round down to the nearest 1-hour clock interval. For example, 12:32:34 is rounded down to 12:00:00.   If you set Period to 5, 10, or 30, the start time of your request is rounded down to the nearest time that corresponds to even 5-, 10-, or 30-second divisions of a minute. For example, if you make a query at (HH:mm:ss) 01:05:23 for the previous 10-second period, the start time of your request is rounded down and you receive data from 01:05:10 to 01:05:20. If you make a query at 15:07:17 for the previous 5 minutes of data, using a period of 5 seconds, you receive data timestamped between 15:02:15 and 15:07:15.  For better performance, specify StartTime and EndTime values that align with the value of the metric's Period and sync up with the beginning and end of an hour. For example, if the Period of a metric is 5 minutes, specifying 12:05 or 12:30 as StartTime can get a faster response from CloudWatch than setting 12:07 or 12:29 as the StartTime.
        end_time(str): The time stamp indicating the latest data to be returned. The value specified is exclusive; results include data points up to the specified time stamp. For better performance, specify StartTime and EndTime values that align with the value of the metric's Period and sync up with the beginning and end of an hour. For example, if the Period of a metric is 5 minutes, specifying 12:05 or 12:30 as EndTime can get a faster response from CloudWatch than setting 12:07 or 12:29 as the EndTime.
        next_token(str, optional): Include this value, if it was returned by the previous GetMetricData operation, to get the next set of data points. Defaults to None.
        scan_by(str, optional): The order in which data points should be returned. TimestampDescending returns the newest data first and paginates when the MaxDatapoints limit is reached. TimestampAscending returns the oldest data first and paginates when the MaxDatapoints limit is reached. Defaults to None.
        max_datapoints(int, optional): The maximum number of data points the request should return before paginating. If you omit this, the default of 100,800 is used. Defaults to None.
        label_options(Dict, optional): This structure includes the Timezone parameter, which you can use to specify your time zone so that the labels of returned data display the correct time for your time zone. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudwatch.metric.data.get(
                ctx, metric_data_queries=value, start_time=value, end_time=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudwatch.metric.data.get metric_data_queries=value, start_time=value, end_time=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudwatch",
        operation="get_metric_data",
        op_kwargs={
            "MetricDataQueries": metric_data_queries,
            "StartTime": start_time,
            "EndTime": end_time,
            "NextToken": next_token,
            "ScanBy": scan_by,
            "MaxDatapoints": max_datapoints,
            "LabelOptions": label_options,
        },
    )


async def put(hub, ctx, namespace: str, metric_data: List) -> None:
    r"""
    **Autogenerated function**

    Publishes metric data points to Amazon CloudWatch. CloudWatch associates the data points with the specified metric. If the specified metric does not exist, CloudWatch creates the metric. When CloudWatch creates a metric, it can take up to fifteen minutes for the metric to appear in calls to ListMetrics. You can publish either individual data points in the Value field, or arrays of values and the number of times each value occurred during the period by using the Values and Counts fields in the MetricDatum structure. Using the Values and Counts method enables you to publish up to 150 values per metric with one PutMetricData request, and supports retrieving percentile statistics on this data. Each PutMetricData request is limited to 40 KB in size for HTTP POST requests. You can send a payload compressed by gzip. Each request is also limited to no more than 20 different metrics. Although the Value parameter accepts numbers of type Double, CloudWatch rejects values that are either too small or too large. Values must be in the range of -2^360 to 2^360. In addition, special values (for example, NaN, +Infinity, -Infinity) are not supported. You can use up to 10 dimensions per metric to further clarify what data the metric collects. Each dimension consists of a Name and Value pair. For more information about specifying dimensions, see Publishing Metrics in the Amazon CloudWatch User Guide. You specify the time stamp to be associated with each data point. You can specify time stamps that are as much as two weeks before the current date, and as much as 2 hours after the current day and time. Data points with time stamps from 24 hours ago or longer can take at least 48 hours to become available for GetMetricData or GetMetricStatistics from the time they are submitted. Data points with time stamps between 3 and 24 hours ago can take as much as 2 hours to become available for for GetMetricData or GetMetricStatistics. CloudWatch needs raw data points to calculate percentile statistics. If you publish data using a statistic set instead, you can only retrieve percentile statistics for this data if one of the following conditions is true:   The SampleCount value of the statistic set is 1 and Min, Max, and Sum are all equal.   The Min and Max are equal, and Sum is equal to Min multiplied by SampleCount.

    Args:
        namespace(str): The namespace for the metric data. To avoid conflicts with Amazon Web Services service namespaces, you should not specify a namespace that begins with AWS/ .
        metric_data(List): The data for the metric. The array can include no more than 20 metrics per call.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudwatch.metric.data.put(
                ctx, namespace=value, metric_data=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudwatch.metric.data.put namespace=value, metric_data=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudwatch",
        operation="put_metric_data",
        op_kwargs={"Namespace": namespace, "MetricData": metric_data},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(
    hub,
    ctx,
    end_time: str,
    profiling_group_name: str,
    start_time: str,
    locale: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of  Recommendation  objects that contain recommendations for a profiling group for a given time period. A list of  Anomaly  objects that contains details about anomalies detected in the profiling group for the same time period is also returned.

    Args:
        end_time(str):  The start time of the profile to get analysis data about. You must specify startTime and endTime. This is specified using the ISO 8601 format. For example, 2020-06-01T13:15:02.001Z represents 1 millisecond past June 1, 2020 1:15:02 PM UTC. .
        locale(str, optional):  The language used to provide analysis. Specify using a string that is one of the following BCP 47 language codes.     de-DE - German, Germany     en-GB - English, United Kingdom     en-US - English, United States     es-ES - Spanish, Spain     fr-FR - French, France     it-IT - Italian, Italy     ja-JP - Japanese, Japan     ko-KR - Korean, Republic of Korea     pt-BR - Portugese, Brazil     zh-CN - Chinese, China     zh-TW - Chinese, Taiwan   . Defaults to None.
        profiling_group_name(str):  The name of the profiling group to get analysis data about. .
        start_time(str):  The end time of the profile to get analysis data about. You must specify startTime and endTime. This is specified using the ISO 8601 format. For example, 2020-06-01T13:15:02.001Z represents 1 millisecond past June 1, 2020 1:15:02 PM UTC. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeguruprofiler.recommendation.get_all(
                ctx, end_time=value, profiling_group_name=value, start_time=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeguruprofiler.recommendation.get_all end_time=value, profiling_group_name=value, start_time=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeguruprofiler",
        operation="get_recommendations",
        op_kwargs={
            "endTime": end_time,
            "locale": locale,
            "profilingGroupName": profiling_group_name,
            "startTime": start_time,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, id_: str, description: str = None, tags: Dict = None
) -> Dict:
    r"""
        **Autogenerated function**

        Creates a new Channel.

        Args:
            description(str, optional): A short text description of the Channel. Defaults to None.
            id_(str): The ID of the Channel. The ID must be unique within the region and it
    cannot be changed after a Channel is created.
    .
            tags(Dict, optional): A collection of tags associated with a resource. Defaults to None.

        Returns:
            Dict

        Examples:
            Call from code:

            .. code-block:: python

                await hub.exec.aws_auto.mediapackage.channel.init.create(ctx, id_=value)

            Call from CLI:

            .. code-block:: bash

                $ idem exec aws_auto.mediapackage.channel.init.create id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediapackage",
        operation="create_channel",
        op_kwargs={"Description": description, "Id": id_, "Tags": tags},
    )


async def delete(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an existing Channel.

    Args:
        id_(str): The ID of the Channel to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediapackage.channel.init.delete(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediapackage.channel.init.delete id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediapackage",
        operation="delete_channel",
        op_kwargs={"Id": id_},
    )


async def describe(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets details about a Channel.

    Args:
        id_(str): The ID of a Channel.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediapackage.channel.init.describe(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediapackage.channel.init.describe id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediapackage",
        operation="describe_channel",
        op_kwargs={"Id": id_},
    )


async def list_all(hub, ctx, max_results: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a collection of Channels.

    Args:
        max_results(int, optional): Upper bound on number of records to return. Defaults to None.
        next_token(str, optional): A token used to resume pagination from the end of a previous request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediapackage.channel.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediapackage.channel.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediapackage",
        operation="list_channels",
        op_kwargs={"MaxResults": max_results, "NextToken": next_token},
    )


async def update(hub, ctx, id_: str, description: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Updates an existing Channel.

    Args:
        description(str, optional): A short text description of the Channel. Defaults to None.
        id_(str): The ID of the Channel to update.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediapackage.channel.init.update(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediapackage.channel.init.update id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediapackage",
        operation="update_channel",
        op_kwargs={"Description": description, "Id": id_},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def disable(hub, ctx, directory_id: str, type_: str) -> Dict:
    r"""
    **Autogenerated function**

    Deactivates LDAP secure calls for the specified directory.

    Args:
        directory_id(str): The identifier of the directory.
        type_(str): The type of LDAP security to enable. Currently only the value Client is supported.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ds.ldap.init.disable(ctx, directory_id=value, type_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ds.ldap.init.disable directory_id=value, type_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ds",
        operation="disable_ldaps",
        op_kwargs={"DirectoryId": directory_id, "Type": type_},
    )


async def enable(hub, ctx, directory_id: str, type_: str) -> Dict:
    r"""
    **Autogenerated function**

    Activates the switch for the specific directory to always use LDAP secure calls.

    Args:
        directory_id(str): The identifier of the directory.
        type_(str): The type of LDAP security to enable. Currently only the value Client is supported.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ds.ldap.init.enable(ctx, directory_id=value, type_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ds.ldap.init.enable directory_id=value, type_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ds",
        operation="enable_ldaps",
        op_kwargs={"DirectoryId": directory_id, "Type": type_},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    access_preview_id: str,
    analyzer_arn: str,
    filter_: Dict = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves a list of access preview findings generated by the specified access preview.

    Args:
        access_preview_id(str): The unique ID for the access preview.
        analyzer_arn(str): The ARN of the analyzer used to generate the access.
        filter_(Dict, optional): Criteria to filter the returned findings. Defaults to None.
        max_results(int, optional): The maximum number of results to return in the response. Defaults to None.
        next_token(str, optional): A token used for pagination of results returned. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.accessanalyzer.access_preview.finding.list_all(
                ctx, access_preview_id=value, analyzer_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.accessanalyzer.access_preview.finding.list_all access_preview_id=value, analyzer_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="accessanalyzer",
        operation="list_access_preview_findings",
        op_kwargs={
            "accessPreviewId": access_preview_id,
            "analyzerArn": analyzer_arn,
            "filter": filter_,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, app_id: str, branch_name: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes a job for a branch of an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        branch_name(str):  The name for the branch, for the job. .
        job_id(str):  The unique ID for the job. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.job.delete(
                ctx, app_id=value, branch_name=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.job.delete app_id=value, branch_name=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="delete_job",
        op_kwargs={"appId": app_id, "branchName": branch_name, "jobId": job_id},
    )


async def get(hub, ctx, app_id: str, branch_name: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Returns a job for a branch of an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        branch_name(str):  The branch name for the job. .
        job_id(str):  The unique ID for the job. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.job.get(
                ctx, app_id=value, branch_name=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.job.get app_id=value, branch_name=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="get_job",
        op_kwargs={"appId": app_id, "branchName": branch_name, "jobId": job_id},
    )


async def start(
    hub,
    ctx,
    app_id: str,
    branch_name: str,
    job_type: str,
    job_id: str = None,
    job_reason: str = None,
    commit_id: str = None,
    commit_message: str = None,
    commit_time: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Starts a new job for a branch of an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        branch_name(str):  The branch name for the job. .
        job_id(str, optional):  The unique ID for an existing job. This is required if the value of jobType is RETRY. . Defaults to None.
        job_type(str):  Describes the type for the job. The job type RELEASE starts a new job with the latest change from the specified branch. This value is available only for apps that are connected to a repository. The job type RETRY retries an existing job. If the job type value is RETRY, the jobId is also required. .
        job_reason(str, optional):  A descriptive reason for starting this job. . Defaults to None.
        commit_id(str, optional):  The commit ID from a third-party repository provider for the job. . Defaults to None.
        commit_message(str, optional):  The commit message from a third-party repository provider for the job. . Defaults to None.
        commit_time(str, optional):  The commit date and time for the job. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.job.start(
                ctx, app_id=value, branch_name=value, job_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.job.start app_id=value, branch_name=value, job_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="start_job",
        op_kwargs={
            "appId": app_id,
            "branchName": branch_name,
            "jobId": job_id,
            "jobType": job_type,
            "jobReason": job_reason,
            "commitId": commit_id,
            "commitMessage": commit_message,
            "commitTime": commit_time,
        },
    )


async def stop(hub, ctx, app_id: str, branch_name: str, job_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Stops a job that is in progress for a branch of an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        branch_name(str):  The name for the branch, for the job. .
        job_id(str):  The unique id for the job. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.job.stop(
                ctx, app_id=value, branch_name=value, job_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.job.stop app_id=value, branch_name=value, job_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="stop_job",
        op_kwargs={"appId": app_id, "branchName": branch_name, "jobId": job_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def cancel(hub, ctx, ledger_name: str, stream_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Ends a given Amazon QLDB journal stream. Before a stream can be canceled, its current status must be ACTIVE. You can't restart a stream after you cancel it. Canceled QLDB stream resources are subject to a 7-day retention period, so they are automatically deleted after this limit expires.

    Args:
        ledger_name(str): The name of the ledger.
        stream_id(str): The UUID (represented in Base62-encoded text) of the QLDB journal stream to be canceled.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.qldb.journal.kinesis_stream.init.cancel(
                ctx, ledger_name=value, stream_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.qldb.journal.kinesis_stream.init.cancel ledger_name=value, stream_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="qldb",
        operation="cancel_journal_kinesis_stream",
        op_kwargs={"LedgerName": ledger_name, "StreamId": stream_id},
    )


async def describe(hub, ctx, ledger_name: str, stream_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns detailed information about a given Amazon QLDB journal stream. The output includes the Amazon Resource Name (ARN), stream name, current status, creation time, and the parameters of the original stream creation request. This action does not return any expired journal streams. For more information, see Expiration for terminal streams in the Amazon QLDB Developer Guide.

    Args:
        ledger_name(str): The name of the ledger.
        stream_id(str): The UUID (represented in Base62-encoded text) of the QLDB journal stream to describe.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.qldb.journal.kinesis_stream.init.describe(
                ctx, ledger_name=value, stream_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.qldb.journal.kinesis_stream.init.describe ledger_name=value, stream_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="qldb",
        operation="describe_journal_kinesis_stream",
        op_kwargs={"LedgerName": ledger_name, "StreamId": stream_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, comment_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the content of a comment made on a change, file, or commit in a repository.   Reaction counts might include numbers from user identities who were deleted after the reaction was made. For a count of reactions from active identities, use GetCommentReactions.

    Args:
        comment_id(str): The unique, system-generated ID of the comment. To get this ID, use GetCommentsForComparedCommit or GetCommentsForPullRequest.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codecommit.comment.init.get(ctx, comment_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codecommit.comment.init.get comment_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codecommit",
        operation="get_comment",
        op_kwargs={"commentId": comment_id},
    )


async def update(hub, ctx, comment_id: str, content: str) -> Dict:
    r"""
    **Autogenerated function**

    Replaces the contents of a comment.

    Args:
        comment_id(str): The system-generated ID of the comment you want to update. To get this ID, use GetCommentsForComparedCommit or GetCommentsForPullRequest.
        content(str): The updated content to replace the existing content of the comment.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codecommit.comment.init.update(
                ctx, comment_id=value, content=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codecommit.comment.init.update comment_id=value, content=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codecommit",
        operation="update_comment",
        op_kwargs={"commentId": comment_id, "content": content},
    )
